"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const scrollIntoView = require('scroll-into-view');
class Actor {
    constructor(root, options) {
        this.name = `${root.tagName}-${root.id}-${root.className}`.toLocaleLowerCase();
        this.root = root;
        this._options = options;
    }
    get options() {
        return this._options;
    }
    getInformation() {
        const rect = this.root.getBoundingClientRect();
        const style = getComputedStyle(this.root);
        return {
            borderRadius: {
                topLeft: style.borderTopLeftRadius,
                topRight: style.borderTopRightRadius,
                bottomLeft: style.borderBottomLeftRadius,
                bottomRight: style.borderBottomRightRadius
            },
            position: {
                x: rect.x - 2 + window.scrollX,
                y: rect.y + window.scrollY,
                width: rect.width - 2,
                height: rect.height
            }
        };
    }
    _isOutOfViewport() {
        const rect = this.root.getBoundingClientRect();
        // checking if all sides are in
        const out = {
            top: rect.top < 0,
            left: rect.left < 0,
            bottom: rect.bottom > (window.innerHeight || document.documentElement.clientHeight),
            right: rect.right > (window.innerWidth || document.documentElement.clientWidth)
        };
        return out.top || out.left || out.bottom || out.right;
    }
    // https://www.npmjs.com/package/scroll-into-view
    scrollIntoView(options) {
        if (this._isOutOfViewport() || options.centerActorOnFocus) {
            scrollIntoView(this.root, {
                time: options.delay
            });
        }
    }
    toString() {
        return `Actor[${this.name}]`;
    }
}
exports.default = Actor;
