"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Spot_1 = require("./Spot");
const OptionBuilder_1 = require("../utils/options/OptionBuilder");
const Sign_1 = require("../sign/Sign");
class Spotlight {
    constructor(options) {
        this.targets = [];
        this.currentActorIndex = -1;
        this.options = OptionBuilder_1.OptionBuilder.buildOptions(options);
        this.isSwitching = false;
        this.spot = new Spot_1.default(this.options);
        this.sign = new Sign_1.Sign(this.options);
    }
    addActors(...actors) {
        for (const a of actors) {
            this.targets.push(a);
        }
    }
    toggleSpot() {
        if (this.isSwitching && this.options.preventFocusSpam) {
            return;
        }
        this.isSwitching = true;
        const on = this.spot.isVisible;
        if (!on) {
            if (this.currentActorIndex === -1) {
                this.currentActorIndex = 0;
            }
            try {
                this._focusActor();
                this.spot.turnOn();
            }
            catch (error) {
                console.error(error);
            }
        }
        else {
            if (this.options.resetOnLightsOff) {
                this.currentActorIndex = -1;
            }
            this.spot.turnOff();
            this.sign.hide();
            setTimeout(() => this.isSwitching = false, this.options.delay);
        }
        return on;
    }
    next() {
        if (this.isSwitching && this.options.preventFocusSpam) {
            return;
        }
        if (this.spot.isVisible || !this.spot.isVisible && this.options.allowHiddenFocus) {
            this.isSwitching = true;
            if (this.currentActorIndex < this.targets.length - 1 && this.currentActorIndex !== -1) {
                this.currentActorIndex++;
            }
            else if (this.currentActorIndex === this.targets.length - 1) {
                if (this.options.loops) {
                    this.currentActorIndex = 0;
                }
                else if (this.options.resetOnEnd) {
                    this.isSwitching = false;
                    this.toggleSpot();
                    this.currentActorIndex = 0;
                    return;
                }
            }
            else if (this.currentActorIndex === -1) {
                this.currentActorIndex =
                    this.targets.length > 1 ? 1 : 0;
            }
            this._focusActor();
        }
    }
    previous() {
        if (this.isSwitching && this.options.preventFocusSpam) {
            return;
        }
        if (this.spot.isVisible || !this.spot.isVisible && this.options.allowHiddenFocus) {
            this.isSwitching = true;
            if (this.currentActorIndex > 0 && this.currentActorIndex !== -1) {
                this.currentActorIndex--;
            }
            else if (this.currentActorIndex === 0 && this.options.loops) {
                this.currentActorIndex = this.targets.length - 1;
            }
            else if (this.currentActorIndex === -1) {
                this.currentActorIndex = this.targets.length - 1;
            }
            this._focusActor();
        }
    }
    _focusActor() {
        if (this.targets.length === 0 || this.currentActorIndex === -1) {
            throw new Error('No actors have been registered, skipping.');
        }
        if (!this.options.allowHiddenFocus)
            this.spot.turnOn();
        this.sign.hide();
        const actor = this.targets[this.currentActorIndex];
        // launching sign content loading before showing it
        this.sign.setContent(actor);
        this.spot.focus(actor)
            .then(() => {
            this.isSwitching = false;
            this.sign.setHTMLMessage(actor);
        });
    }
}
exports.default = Spotlight;
